//Formulary for CRUD actions
import React, {useState, useEffect} from 'react';
import {
  Text,
  Modal,
  SafeAreaView,
  View,
  TextInput,
  ScrollView,
  Alert,
  TouchableOpacity,
} from 'react-native';
import DatePicker from 'react-native-date-picker';
import {Picker} from '@react-native-picker/picker';
import {idGen} from '../helpers/Index';
import styles from '../styles/Index';
import Icon from 'react-native-vector-icons/MaterialIcons';

//Define modal close icon
const cancel = <Icon name="cancel" size={36} color="#ffa291" />;

//Props from parent
const CropForm = ({
  cropModal,
  setCropModal,
  cultures,
  setCultures,
  culture: cultureObj,
  setCulture,
  saveCultureStorage,
}) => {
  //Declare state culture properties and initialize without value
  const [id, setId] = useState('');
  const [fieldName, setFieldName] = useState('');
  const [fieldUbication, setFieldUbication] = useState('');
  const [fieldSize, setFieldSize] = useState('');
  const [fieldType, setFieldType] = useState('');
  const [cultureType, setCultureType] = useState('');
  const [fieldCultureDate, setFieldCultureDate] = useState(new Date());
  const [fieldStatus, setFieldStatus] = useState('');
  const [fieldNotes, setFieldNotes] = useState('');

  //If already exists or change the culture, set properties
  useEffect(() => {
    if (Object.keys(cultureObj).length > 0) {
      setId(cultureObj.id);
      setFieldName(cultureObj.fieldName);
      setFieldUbication(cultureObj.fieldUbication);
      setFieldSize(cultureObj.fieldSize);
      setFieldType(cultureObj.fieldType);
      setCultureType(cultureObj.cultureType);
      setFieldCultureDate(new Date(cultureObj.fieldCultureDate));
      setFieldStatus(cultureObj.fieldStatus);
      setFieldNotes(cultureObj.fieldNotes);
    }
  }, [cultureObj]);

  //Create new culture
  const addCulture = () => {
    if (
      [
        fieldName,
        fieldUbication,
        fieldSize,
        fieldType,
        cultureType,
        fieldCultureDate,
      ].includes('')
    ) {
      // Validation required fields
      Alert.alert('Error', 'Por favor, rellene todos los campos obligatorios');
      return;
    }

    //Construct the new culture with formerly obtained data
    const newCulture = {
      fieldName,
      fieldUbication,
      fieldSize,
      fieldType,
      cultureType,
      fieldCultureDate,
      fieldStatus,
      fieldNotes,
    };

    //Check if id already exists to modify item or if not, create new culture
    if (id) {
      newCulture.id = id;
      const updateCultures = cultures.map(cultureState =>
        cultureState.id === newCulture.id ? newCulture : cultureState,
      );

      setCultures(updateCultures);
      saveCultureStorage(JSON.stringify(updateCultures));
      setCulture({});
    } else {
      newCulture.id = idGen();
      setCultures([...cultures, newCulture]);
      saveCultureStorage(JSON.stringify([...cultures, newCulture]));
    }

    //Toggle modal when create or update action finished
    setCropModal(!cropModal);

    //Reset fields
    setId('');
    setFieldName('');
    setFieldUbication('');
    setFieldSize('');
    setFieldType('');
    setCultureType('');
    setFieldCultureDate(new Date());
    setFieldStatus('');
    setFieldNotes('');
  };

  return (
    //Use modal to show the form
    <Modal animationType="slide" visible={cropModal}>
      <SafeAreaView style={styles.modalContent}>
        {/* Set the title according function type */}
        <Text style={styles.modalTitle}>
          {cultureObj.id ? 'Editar Cultivo' : 'Nuevo Cultivo'}
        </Text>
        {/* Disable modal and restore data when close */}
        <View style={styles.modalContainer}>
          <TouchableOpacity
            style={styles.modalClose}
            onLongPress={() => {
              setCropModal(!cropModal);
              setCulture({});
              setId('');
              setFieldName('');
              setFieldUbication('');
              setFieldSize('');
              setFieldType('');
              setCultureType('');
              setFieldCultureDate(new Date());
              setFieldStatus('');
              setFieldNotes('');
            }}
          >
            <Text>{cancel}</Text>
          </TouchableOpacity>
        </View>
        {/* View modal and show form */}
        <ScrollView>
          <View style={styles.formField}>
            <Text style={styles.formLabel}>Nombre del campo:</Text>
            <TextInput
              style={styles.formInput}
              placeholder="Nombre del campo"
              placeholderTextColor={'#666'}
              value={fieldName}
              onChangeText={setFieldName}
            />
          </View>
          <View style={styles.formField}>
            <Text style={styles.formLabel}>Ubicación:</Text>
            <TextInput
              style={styles.formInput}
              placeholder="Lugar, zona, dirección, etc."
              placeholderTextColor={'#666'}
              value={fieldUbication}
              onChangeText={setFieldUbication}
            />
          </View>
          <View style={styles.formField}>
            <Text style={styles.formLabel}>Tamaño:</Text>
            <TextInput
              style={styles.formInput}
              keyboardType="number-pad"
              placeholder="Área cultivable (en metros cuadrados)"
              placeholderTextColor={'#666'}
              value={fieldSize}
              onChangeText={setFieldSize}
            />
          </View>

          <View style={styles.formField}>
            <Text style={styles.formLabel}>Tipo de cultivo:</Text>
            <View style={styles.formDatePicker}>
              <Picker
                selectedValue={fieldType}
                onValueChange={value => {
                  setFieldType(value);
                }}
              >
                <Picker.Item label="-- Especie --" value="" />
                <Picker.Item label="Cereales" value="Cereales" />
                <Picker.Item label="Leguminosas" value="Leguminosas" />
                <Picker.Item label="Oleaginosas" value="Oleaginosas" />
                <Picker.Item label="Hortalizas" value="Hortalizas" />
                <Picker.Item label="Tubérculos" value="Tubérculos" />
                <Picker.Item label="Frutales" value="Frutales" />
                <Picker.Item label="Pastos" value="Pastos" />
              </Picker>
            </View>
          </View>

          <View style={styles.formField}>
            <Text style={styles.formLabel}>Variedad:</Text>
            <TextInput
              style={styles.formInput}
              placeholder="Variedad de cultivo"
              placeholderTextColor={'#666'}
              value={cultureType}
              onChangeText={setCultureType}
            />
          </View>

          <View style={styles.formField}>
            <Text style={styles.formLabel}>Fecha de Siembra:</Text>
            <View style={styles.formDatePicker}>
              <DatePicker
                date={fieldCultureDate}
                locale="es"
                mode="date"
                onDateChange={date => setFieldCultureDate(date)}
                androidVariant="nativeAndroid"
              />
            </View>
          </View>
          <View style={styles.formField}>
            <Text style={styles.formLabel}>Estado de cultivo:</Text>
            <TextInput
              style={styles.formInput}
              keyboardType="number-pad"
              placeholder="Progreso del cultivo en % (opcional)"
              placeholderTextColor={'#666'}
              value={fieldStatus}
              onChangeText={setFieldStatus}
            />
          </View>
          <View style={styles.formField}>
            <Text style={styles.formLabel}>Notas:</Text>
            <TextInput
              style={[styles.formInput, styles.formTextBox]}
              placeholder="Apuntes y observaciones (opcional)"
              placeholderTextColor={'#666'}
              value={fieldNotes}
              onChangeText={setFieldNotes}
              multiline={true}
              numberOfLines={5}
            />
          </View>
          {/* Validate form and add or update culture */}
          <TouchableOpacity
            style={styles.btnModAdd}
            onPress={() => addCulture()}
          >
            <Text style={styles.btnModAddText}>
              {cultureObj.id ? 'Modificar Datos' : 'Añadir Cultivo'}
            </Text>
          </TouchableOpacity>
        </ScrollView>
      </SafeAreaView>
    </Modal>
  );
};

export default CropForm;
