//Stock form for CRUD actions
import React, {useState, useEffect} from 'react';
import {
  Text,
  Modal,
  SafeAreaView,
  View,
  TextInput,
  ScrollView,
  Alert,
  TouchableOpacity,
} from 'react-native';
import DatePicker from 'react-native-date-picker';
import {Picker} from '@react-native-picker/picker';
import {idGen} from '../helpers/Index';
import styles from '../styles/Index';
import Icon from 'react-native-vector-icons/MaterialIcons';

//Declare close modal icon
const cancel = <Icon name="cancel" size={36} color="#ffa291" />;

//Props from view and define new empty state constants for each
const StockForm = ({
  livestockModal,
  setLivestockModal,
  livestock,
  setLivestock,
  animal: animalObj,
  setAnimal,
  saveLivestockStorage,
}) => {
  const [id, setId] = useState('');
  const [stockName, setStockName] = useState('');
  const [stockId, setStockId] = useState('');
  const [stockSpecie, setStockSpecie] = useState('');
  const [stockBreed, setStockBreed] = useState('');
  const [stockBirthDate, setStockBirthDate] = useState(new Date());
  const [stockWeight, setStockWeight] = useState('');
  const [stockVaccine, setStockVaccine] = useState('');
  const [stockMedication, setStockMedication] = useState('');
  const [stockNotes, setStockNotes] = useState('');

  useEffect(() => {
    //If there are any item or change state, set properties
    if (Object.keys(animalObj).length > 0) {
      setId(animalObj.id);
      setStockName(animalObj.stockName);
      setStockId(animalObj.stockId);
      setStockSpecie(animalObj.stockSpecie);
      setStockBreed(animalObj.stockBreed);
      setStockBirthDate(new Date(animalObj.stockBirthDate));
      setStockWeight(animalObj.stockWeight);
      setStockVaccine(animalObj.stockVaccine);
      setStockMedication(animalObj.stockMedication);
      setStockNotes(animalObj.stockNotes);
    }
  }, [animalObj]);

  //Add new stock
  const addStock = () => {
    // Validation function for required arguments
    if (
      [stockName, stockId, stockSpecie, stockBreed, stockBirthDate].includes('')
    ) {
      Alert.alert('Error', 'Por favor, rellene todos los campos obligatorios');
      return;
    }

    //New stock properties structure
    const newStock = {
      stockName,
      stockId,
      stockSpecie,
      stockBreed,
      stockBirthDate,
      stockWeight,
      stockVaccine,
      stockMedication,
      stockNotes,
    };

    // Check if exists stock for update or create a new one
    if (id) {
      newStock.id = id;
      const updateStock = livestock.map(stockState =>
        stockState.id === newStock.id ? newStock : stockState,
      );

      setLivestock(updateStock);
      saveLivestockStorage(JSON.stringify(updateStock));
      setAnimal({});
    } else {
      newStock.id = idGen();
      setLivestock([...livestock, newStock]);
      saveLivestockStorage(JSON.stringify([...livestock, newStock]));
    }

    //Toggle form modal when finished operations
    setLivestockModal(!livestockModal);

    //Reset fields
    setId('');
    setStockName('');
    setStockId('');
    setStockSpecie('');
    setStockBreed('');
    setStockBirthDate(new Date());
    setStockWeight('');
    setStockVaccine('');
    setStockMedication('');
    setStockNotes('');
  };

  return (
    //Set visible for modal
    <Modal animationType="slide" visible={livestockModal}>
      <SafeAreaView style={styles.modalContent}>
        {/* Set title according to function */}
        <Text style={styles.modalTitle}>
          {animalObj.id ? 'Editar Animal' : 'Nuevo Animal'}
        </Text>
        <View style={styles.modalContainer}>
          {/* Disable modal and restore data when close */}
          <TouchableOpacity
            style={styles.modalClose}
            onLongPress={() => {
              setLivestockModal(!livestockModal);
              setAnimal({});
              setId('');
              setStockName('');
              setStockId('');
              setStockSpecie('');
              setStockBreed('');
              setStockBirthDate(new Date());
              setStockWeight('');
              setStockVaccine('');
              setStockMedication('');
              setStockNotes('');
            }}
          >
            <Text>{cancel}</Text>
          </TouchableOpacity>
        </View>
        {/* Enable modal and show fields */}
        <ScrollView>
          <View style={styles.formField}>
            <Text style={styles.formLabel}>Nombre del animal:</Text>
            <TextInput
              style={styles.formInput}
              placeholder="Nombre del animal"
              placeholderTextColor={'#666'}
              value={stockName}
              onChangeText={setStockName}
            />
          </View>
          <View style={styles.formField}>
            <Text style={styles.formLabel}>Identificación:</Text>
            <TextInput
              style={styles.formInput}
              placeholder="Identificación"
              placeholderTextColor={'#666'}
              value={stockId}
              onChangeText={setStockId}
            />
          </View>

          <View style={styles.formField}>
            <Text style={styles.formLabel}>Especie:</Text>
            <View style={styles.formDatePicker}>
              <Picker
                selectedValue={stockSpecie}
                onValueChange={value => {
                  setStockSpecie(value);
                }}
              >
                <Picker.Item label="-- Tipo --" value="" />
                <Picker.Item label="Bovina" value="Bovina" />
                <Picker.Item label="Ovina" value="Ovina" />
                <Picker.Item label="Caprina" value="Caprina" />
                <Picker.Item label="Equina" value="Equina" />
                <Picker.Item label="Otra" value="Otra" />
              </Picker>
            </View>
          </View>

          <View style={styles.formField}>
            <Text style={styles.formLabel}>Raza:</Text>
            <TextInput
              style={styles.formInput}
              placeholder="Raza"
              placeholderTextColor={'#666'}
              value={stockBreed}
              onChangeText={setStockBreed}
            />
          </View>
          <View style={styles.formField}>
            <Text style={styles.formLabel}>Fecha de Nacimiento:</Text>
            <View style={styles.formDatePicker}>
              <DatePicker
                date={stockBirthDate}
                locale="es"
                mode="date"
                onDateChange={date => setStockBirthDate(date)}
                androidVariant="nativeAndroid"
              />
            </View>
          </View>

          <View style={styles.formField}>
            <Text style={styles.formLabel}>Peso:</Text>
            <TextInput
              style={styles.formInput}
              keyboardType="number-pad"
              placeholder="Peso (Opcional)"
              placeholderTextColor={'#666'}
              value={stockWeight}
              onChangeText={setStockWeight}
            />
          </View>
          <View style={styles.formField}>
            <Text style={styles.formLabel}>Vacunación y tratamientos:</Text>
            <TextInput
              style={[styles.formInput, styles.formTextBox]}
              placeholder="Vacunación y tratamientos (Opcional)"
              placeholderTextColor={'#666'}
              value={stockVaccine}
              onChangeText={setStockVaccine}
              multiline={true}
              numberOfLines={3}
            />
          </View>
          <View style={styles.formField}>
            <Text style={styles.formLabel}>Medicación y enfermedades:</Text>
            <TextInput
              style={[styles.formInput, styles.formTextBox]}
              placeholder="Medicación y enfermedades (Opcional)"
              placeholderTextColor={'#666'}
              value={stockMedication}
              onChangeText={setStockMedication}
              multiline={true}
              numberOfLines={3}
            />
          </View>
          <View style={styles.formField}>
            <Text style={styles.formLabel}>Notas:</Text>
            <TextInput
              style={[styles.formInput, styles.formTextBox]}
              placeholder="Apuntes y observaciones (Opcional)"
              placeholderTextColor={'#666'}
              value={stockNotes}
              onChangeText={setStockNotes}
              multiline={true}
              numberOfLines={5}
            />
          </View>
          {/* Validate form and add or update stock */}
          <TouchableOpacity style={styles.btnModAdd} onPress={() => addStock()}>
            <Text style={styles.btnModAddText}>
              {animalObj.id ? 'Modificar Datos' : 'Añadir Animal'}
            </Text>
          </TouchableOpacity>
        </ScrollView>
      </SafeAreaView>
    </Modal>
  );
};

export default StockForm;
